/****************************************************************************
*
*     Header Filename:  Simulation_Scheduler.cpp
*     Module Name:      Simulation_Scheduler
*     Description:      Manages simulation processing while there is no user input 
*                       to be processed by the user interface
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

#include    <gtkmm.h>

//------------------  Simulation Subsystem -----------------------------------------------------------------------
#include "Scenario_Manager.h"
#include "Simulation_Scheduler.h"
//----------------------------------------------------------------------------------------------------------------

Simulation_Scheduler* Simulation_Scheduler::Instance_Ptr = 0;
//================================================================================================================

void  Simulation_Scheduler::Disable_Simulation_Processing()

//  Stop the scheduler from running the simulation while the GUI is idle

{
    Simulation_Processing_Enabled = false;
}
//----------------------------------------------------------------------------------------------------------------

void  Simulation_Scheduler::Enable_Simulation_Processing()

//  Connect GTKMM to the simulation routine to be run 
//  while the GUI is idle and enable calling of the routine

{
    Glib::signal_idle().connect( sigc::mem_fun(*this, 
                                            &Simulation_Scheduler::Handle_Processing_Time_Available_Notification));
    


Simulation_Processing_Enabled = true;
}
//----------------------------------------------------------------------------------------------------------------
Simulation_Scheduler::Simulation_Scheduler()

{
    Simulation_Processing_Enabled = false;
};
//----------------------------------------------------------------------------------------------------------------

void  Simulation_Scheduler::Make()

//  Create the singleton object

{
    Assert(Instance_Ptr == 0,"Simulation_Scheduler::Make -  Invalid call; instance already been created");
    CREATE_OBJECT(Simulation_Scheduler, Instance_Ptr = new Simulation_Scheduler);   
}
//----------------------------------------------------------------------------------------------------------------

bool  Simulation_Scheduler::SimProc_IsEnabled()

//  Return whether or not simulation processing is enabled, 

{
    return Simulation_Processing_Enabled ;
}
//----------------------------------------------------------------------------------------------------------------

bool  Simulation_Scheduler::Handle_Processing_Time_Available_Notification()

//  Handler invoked when the GUI becomes idle (if it is connected)

{
    if (Simulation_Processing_Enabled)
    THEN 
        int Time_To_Run = 50;  //  Milliseconds
        Scenario_Manager::Get_Ptr()->Run_Current_Scenario_Series(Time_To_Run);
        return true; 
    ELSE
        return false;   // This will disconnect the handler 
    ENDIF
}
