/****************************************************************************
*
*	  Module Filename:  Simulation_Statistics.cpp
*	  Module Name:      Simulation_Statistics
*	  Description:      Records statistics gathered during running of simulation
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include  "globldef.h"
#include  "Library_Headers.h"

//--------------  Statistics  Subsystem  --------------------------------------------------------------------------

#include  "Simulation_Statistics.h"
#include  "Indexed_Series.h"
//-----------------------------------------------------------------------------------------------------------------

Simulation_Statistics*  Simulation_Statistics::Instance_Ptr = 0;
//-----------------------------------------------------------------------------------------------------------------

//  void  Simulation_Statistics::Clear()
//  
// //   Clear the Simulation Statistics
// 
// {
//     for (bool Continue_Flag1 = Iter_To_First_Ind_Series_List();
//          Continue_Flag1 == true;
//          Continue_Flag1 = Iter_To_Next_Ind_Series_List())
//     BEGIN
//         for (bool Continue_Flag2 = Iter_To_First_Ind_Series();
//             Continue_Flag2 == true;
//             Continue_Flag2 = Iter_To_Next_Ind_Series())
//         BEGIN
//             Indexed_Series* Series_Ptr = Get_Current_Indexed_Series_Ptr();
//             Series_Ptr->Clear();
//             DELETE_OBJECT (Indexed_Series, Series_Ptr);
//         END
//     END
//     List_Of_Indexed_Series_Lists.clear();
// }
//---------------------------------------------------------------------------------------------------------------

void  Simulation_Statistics::Clear_Series_List(string List_Name)

//  Delete all of the Indexed Series in the specified Series List.
//  Clear, but do not delete, the list
//  Do not delete the list itself

{
    TRACE("Simulation_Statistics::Clear_Series_List");

    Set_Current_Ind_Series_List(List_Name);

    for (bool Cont_Fl = Iter_To_First_Ind_Series();
        Cont_Fl == true;
        Cont_Fl = Iter_To_Next_Ind_Series())
    BEGIN
        Indexed_Series* Series_Pt = Get_Current_Indexed_Series_Ptr();
        Series_Pt->Clear();
        DELETE_OBJECT (Indexed_Series, Series_Pt);
    END

//  Now clear the list
    List_Of_Indexed_Series_Lists[List_Name].second.clear();
}

//----------------------------------------------------------------------------------------------------------------
Indexed_Series*    Simulation_Statistics::Create_Indexed_Series(string Indexed_Series_Name,
                                                                string Index_Name, 
                                                                string Value_Name,
                                                                string Series_List,
                                                                bool No_Entry_Means_Zero)
//  Create a new Indexed_Series for the specified Series_Group
//  Add the Series to the List of Lists of Indexed Series.

{
    TRACE("Simulation_Statistics::Create_Indexed_Series");

    Assert(List_Of_Indexed_Series_Lists.find(Series_List) not_eq List_Of_Indexed_Series_Lists.end(),
           "Simulation_Statistics::Create_Indexed_Series - Attempt to create an Indexed_Series for a Series_List that does not exist");
           
    CREATE_OBJECT(Indexed_Series, Indexed_Series* Series_Ptr = new Indexed_Series(Indexed_Series_Name,  
                                                                                  Series_List,
                                                                                  Index_Name, 
                                                                                  Value_Name, 
                                                                                  No_Entry_Means_Zero));
    Set_Current_Ind_Series_List(Series_List);
           
    (*List_Of_Ind_Series_Lists_Iter).second.second.push_back(Series_Ptr);
    return Series_Ptr;
}
//-----------------------------------------------------------------------------------------------------------------

void   Simulation_Statistics::Create_Indexed_Series_List(string Series_List_Name,
                                                         bool Is_For_User_Access_Fl)

//  Add a new list of Indexed_Series 

{
    TRACE("Create_Indexed_Series_List");
    pair <string, pair<bool, vector<Indexed_Series*> > > P;
    P.first = Series_List_Name;
    vector<Indexed_Series*> Vec;
    P.second.first = Is_For_User_Access_Fl;
    P.second.second = Vec;
    List_Of_Indexed_Series_Lists.insert(P);
}
//----------------------------------------------------------------------------------------------------------------
void  Simulation_Statistics::Create_And_Populate_Manhattn_Dist_Ser
                                (string Manhattan_Series_Name,      // Name of the Manhattan Series to be
                                                                    // created and populated
                                 string Manhattan_Series_List_Name, // The name of the list of Manhattan Series 
                                                                    // to whch the Manhattan series will belong
                                 string Base_Data_Series_List_Name, // Name of the list of base data series from 
                                                                    // which the Manhattan series is to be produced
                                 double Last_Vector_Index,          // Index of the last base data series 
                                 int Vector_Index_Increment)        // Index increment between the base data series

//  Create an Indexed series and populate it with Manhattan distances between the
//  vectors stored in the list of base data series.
//  The 'vectors' are slices through the base data series. The set of values of teh 
//  base data series for a particular index form a vector.

{
    TRACE("Simulation_Statistics::Create_And_Populate_Manhattn_Dist_Ser");

//  Create an Indexed Series to hold the Manhattan Distance Series data    
    Indexed_Series* Manhattan_Series_Ptr = Create_Indexed_Series(Manhattan_Series_Name, 
                                                                 string("Time"), 
                                                                 string("Manhattan Distance"),
                                                                 Manhattan_Series_List_Name,
                                                                 true);
//  Select the base data list of series
    Set_Current_Ind_Series_List(Base_Data_Series_List_Name);

//  Loop over base data Vectors    
    for (double Current_Vector_Index = 0;
         Current_Vector_Index <= Last_Vector_Index;
         Current_Vector_Index += Vector_Index_Increment)
    BEGIN 
        double Manhattan_Distance = 0;
        double Current_Value;
        double Last_Value;
        double Manhattan_Length = 0;
   
//      For the current Vector, loop over all Values
        for (bool Continue_Flag1 = Iter_To_First_Ind_Series();
            Continue_Flag1 == true;
            Continue_Flag1 = Iter_To_Next_Ind_Series())
        BEGIN
            Indexed_Series*  Current_Series_Ptr = Get_Current_Indexed_Series_Ptr();
            Current_Series_Ptr->Get_Value_For_Index(Current_Vector_Index, Current_Value);
            Current_Series_Ptr->Get_Value_For_Index(Last_Vector_Index, Last_Value);       
            Manhattan_Distance += abs(Last_Value - Current_Value);   
            Manhattan_Length += Current_Value;   
        END        
        Manhattan_Series_Ptr->Add_Value(Current_Vector_Index, Manhattan_Distance);
    END
}
//----------------------------------------------------------------------------------------------------------------

void  Simulation_Statistics::Delete_Series(Indexed_Series* Series_Pt)

//  Delete the specified Indexed Series, and erase it from the 
//  appropriate list of Indexed Series

{
    TRACE("Simulation_Statistics::Delete");
    string ListName = Series_Pt->Get_Series_List_Name();
    string SeriesName = Series_Pt->Get_Indexed_Series_Name();

//  Find the Series list in which this series appears
    map <string,pair<bool, vector<Indexed_Series*> > >::iterator Lists_Iter;
    Lists_Iter = List_Of_Indexed_Series_Lists.find(ListName);
    Assert (Lists_Iter != List_Of_Indexed_Series_Lists.end(),
            "Simulation_Statistics::Delete - Cannot find Series List");

//  Now find the Indexed Series in the list
    vector<Indexed_Series*>::iterator List_Iter;
    vector<Indexed_Series*>& IndSer = (*Lists_Iter).second.second; 
    for (List_Iter =  IndSer.begin();
         List_Iter != IndSer.end();
         ++List_Iter)
    BEGIN
        if ((*List_Iter) == Series_Pt) break;
    ENDIF
    Assert(List_Iter != IndSer.end(), "Simulation_Statistics::Delete - Cannot find Series");
    Assert((*List_Iter)->Get_Indexed_Series_Name() == SeriesName, 
         "Simulation_Statistics::Delete - Series Name mismatch");

    DELETE_OBJECT(Indexed_Series, (*List_Iter));
    IndSer.erase(List_Iter);
}
//----------------------------------------------------------------------------------------------------------------

Simulation_Statistics*  Simulation_Statistics::Get_Ptr()

//  Get a pointer to the Simulation_Statistics object. It is a fatal error is the Simulation_Statistics 
//  object has not yet been initialised.

{
    Assert(Instance_Ptr not_eq 0,"Simulation_Statistics::Get_Ptr - Called before an instance has been created");
    return (Instance_Ptr);
}
//---------------------------------------------------------------------------------------------------------------
        
    Indexed_Series*    Simulation_Statistics::Get_Current_Indexed_Series_Ptr()

//  Return a pointer to the current Series

{
    Assert (not (*List_Of_Ind_Series_Lists_Iter).second.second.empty(),"Simulation_Statistics::Get_Current_Indexed_Series_Ptr - No Statistics exist for the current Series_List");
    return (*List_Of_Ind_Series_Lists_Iter).second.second[Current_Indexed_Series_Index];
}
//-----------------------------------------------------------------------------------------------------------------

bool   Simulation_Statistics::Get_Curr_SerList_Usr_Access()

//  Return the user accessibility for the current series list
//  true = serise list is accessible to user

{
    TRACE("Simulation_Statistics::Get_Curr_SerList_Usr_Access");

    Assert(!List_Of_Indexed_Series_Lists.empty(), "Simulation_Statistics::Get_Curr_SerList_Usr_Access - No Indexed Series Lists");
    return (*List_Of_Ind_Series_Lists_Iter).second.first;
}
//-----------------------------------------------------------------------------------------------------------------

string Simulation_Statistics::Get_Curr_SerList_Name()

//  Get the name of the current Indexed Series List

{
    TRACE("Simulation_Statistics::Get_Curr_SerList_Name");

    Assert(!List_Of_Indexed_Series_Lists.empty(), "Simulation_Statistics::Get_Curr_SerList_Name - No Indexed Series Lists");
    return (*List_Of_Ind_Series_Lists_Iter).first;
}
//----------------------------------------------------------------------------------------------------------------

string    Simulation_Statistics::Get_Default_SerList_Nm()

//  Return the name of the default Series List

{
    return Default_SerList_Tx_;
}

//----------------------------------------------------------------------------------------------------------------

bool Simulation_Statistics::Iter_To_First_Ind_Series()

//  Set the Series Index to point to the first Indexed Series for the current Series_List. 
//  Return false if there are no Series

{
    TRACE("Iter_To_First_Ind_Series");
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
           "Simulation_Statistics::Iter_To_First_Ind_Series - No valid series currently selected");

    if(not (*List_Of_Ind_Series_Lists_Iter).second.second.empty())
    THEN
        Current_Indexed_Series_Index = 0;
        return true;
    ELSE
        return false;
    ENDIF
}   
//-------------------------------------------------------------------------------------------------------------- 

bool    Simulation_Statistics::Iter_To_Next_Ind_Series()

//  Move the Simulation_Statistics's index for the current Series_List to point 
//  to the next Indexed Series. Return false if already
//  on last item. Fail if there are no Series

{
    TRACE("Iter_To_Next_Ind_Series");
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
           "Simulation_Statistics::Iter_To_Next_Ind_Series - No valid series currently selected");

    if(++Current_Indexed_Series_Index < (*List_Of_Ind_Series_Lists_Iter).second.second.size())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//-----------------------------------------------------------------------------------------------------------------


bool Simulation_Statistics::Iter_To_Last_Ind_Series()

//  Set the Series Index to identify the last Indexed Series for the current Series_List. 
//  Return false if there are no Series

{
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
           "Simulation_Statistics::Iter_To_Last_Ind_Series - No valid series currently selected");

   if(not (*List_Of_Ind_Series_Lists_Iter).second.second.empty())
    THEN
        Current_Indexed_Series_Index = (*List_Of_Ind_Series_Lists_Iter).second.second.size() - 1;
        return true;
    ELSE
        return false;
    ENDIF
}   
//-----------------------------------------------------------------------------------------------------------------

bool    Simulation_Statistics::Iter_To_Prev_Ind_Series()

//  Move the Simulation_Statistics's index for the current Series_List to point 
//  to the previous Indexed Series. Return false if there is no previous series

{
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
           "Simulation_Statistics::Iter_To_Prev_Ind_Series - No valid series currently selected");

    if(--Current_Indexed_Series_Index >= 0)
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}



//-----------------------------------------------------------------------------------------------------------------

bool    Simulation_Statistics::Iter_To_First_Ind_Series_List()

//  Select the first List of Indexed Series Lists. Return false if there are no Lists

{
    TRACE("Iter_To_First_Ind_Series_List");
    List_Of_Ind_Series_Lists_Iter = List_Of_Indexed_Series_Lists.begin();
    if (List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//------------------------------------------------------------------------------------------------------------------      
        
bool    Simulation_Statistics::Iter_To_Next_Ind_Series_List()

//  Select the next List of Indexed Series Lists. Return false if there are no more Lists
 
{
    TRACE("Iter_To_Next_Ind_Series_List");
    if(++List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end())
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//-------------------------------------------------------------------------------------------------------------- 

void  Simulation_Statistics::Make()

//  Initialise the single instance Simulation_Statistics object. Its a fatal error if one already exists.

{
    Assert(Instance_Ptr == 0,"Simulation_Statistics::Make -  Invalid call; Simulation_Statistics object has already been created");    
    CREATE_OBJECT(Simulation_Statistics, Instance_Ptr = new Simulation_Statistics);
}
//-----------------------------------------------------------------------------------------------------------------
void    Simulation_Statistics::Set_Current_Ind_Series_List(string List_Name)

//  Select the list of Indexed Series with the specified name

{
    TRACE("Simulation_Statistics::Set_Current_Ind_Series_List");
    List_Of_Ind_Series_Lists_Iter = List_Of_Indexed_Series_Lists.find(List_Name);
    
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
    "Simulation_Statistics::Set_Current_Ind_Series_List - Invalid List_Name");
}
//----------------------------------------------------------------------------------------------------------------

void    Simulation_Statistics::Set_Default_SerList(string Series_List)

//  Set the Default Series list. Can only be done once, and the Series List must exist

{
    TRACE("Simulation_Statistics::Set_Initial_SerList");
    Assert (Default_SerList_Tx_  == string(""), "Simulation_Statistics::Set_Default_SerList - Attempt to re-initialise default Series List");
    Default_SerList_Tx_ = Series_List;

    List_Of_Ind_Series_Lists_Iter = List_Of_Indexed_Series_Lists.find(Series_List);
    Assert(List_Of_Ind_Series_Lists_Iter not_eq List_Of_Indexed_Series_Lists.end(),
    "Simulation_Statistics::Set_Initial_SerList - Invalid List_Name");
}
//---------------------------------------------------------------------------------------------------------------

Simulation_Statistics::Simulation_Statistics()

//  Constructor: Initialise Series Index

{
    Current_Indexed_Series_Index = 0;
    Default_SerList_Tx_ = string("");
//    Collection_Interval = 500;
}
//----------------------------------------------------------------------------------------------------------------


